@@ -1,15 +1,15 @@
 /*
  * Copyright (c) Paul Kinnucan 1998, 1999, 2000, 2001. All Rights Reserved.
  *
- * $Revision: 1.6 $ 
- * $Date: 2003/01/18 05:48:41 $ 
+ * $Revision: 1.14 $ 
+ * $Date: 2004/03/18 05:28:36 $ 
  *
  * MethodOverrideFactory is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2, or (at
  * your option) any later version.
  *
- * InterfaceFactory is distributed in the hope that it will be useful,
+ * MethodOverrideFactory is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
@@ -35,7 +35,7 @@ import jde.util.DynamicClassLoader;
  * superclass.
  *
  * @author Paul Kinnucan
- * @version $Revision: 1.6 $
+ * @version $Revision: 1.14 $
  */
 
 public class MethodOverrideFactory extends MethodFactory
@@ -63,21 +63,53 @@ public class MethodOverrideFactory extends MethodFactory
     super(factory);
   }
 
-  public Vector getMethods(Class cl, String methodName) {
-
-    Method [] methods = cl.getDeclaredMethods();
-   
-    Vector m = new Vector();
-
-    for (int i = 0; i < methods.length; ++i) 
-      if (methods[i].getName().equals(methodName)) 
-	m.addElement(methods[i]);  
+  /**
+   * Return a <code>Vector</code> containing all <code>Method</code>
+   * objects for the argument class and methodName which can be
+   * overridden.
+   *
+   * @param cl the <code>Class</code> for which all methods that can
+   * be overridden will be returned.
+   * @param methodName the name of the Method to be returned.
+   * @return a <code>Vector</code> of <code>Method</code> objects.
+   */
+  private Vector getOverrideableMethods(Class cl, String methodName) {
     
-    return m;
+    Vector m = new Vector();
+    Class baseClass = cl;
 
+    while (baseClass != null) {
+      Method[] methods = baseClass.getDeclaredMethods();
+      for (int i = 0; i < methods.length; i++) {
+        if (methods[i].getName().equals(methodName)) {
+          int methodModifiers = methods[i].getModifiers();
+          if ((Modifier.isPublic(methodModifiers) ||
+               Modifier.isProtected(methodModifiers)) &&
+              !Modifier.isFinal(methodModifiers))
+            m.addElement(methods[i]);
+        }
+      }
+      baseClass = baseClass.getSuperclass();
+    }
+
+    return m;
   }
 
 
+
+  /**
+   * Get a list of signatures for all methods of a specified name defined
+   * or inherited by a specified class. The signatures returned by this method
+   * including only the types of the method parameters as the generated names
+   * are unnecessary for signature matching. This method prints the signature
+   * list to standard out as a Lisp list form.
+   *
+   * @param baseClassName a <code>String</code> value that specifies the name of
+   * the class that defines or inherits the methods whose signatures are to be
+   * returned.
+   * @param methodName a <code>String</code> value that specifies the name of the
+   * methods whose signatures are to be returned.
+   */
   public static void getCandidateSignatures(String baseClassName, String methodName) {
 
     if (overrideFactory == null) 
@@ -91,38 +123,31 @@ public class MethodOverrideFactory extends MethodFactory
     try {
 
       DynamicClassLoader dcl = new DynamicClassLoader();
-      Class baseClass = dcl.loadClass(baseClassName);
+      Vector methods = overrideFactory.getOverrideableMethods
+          (dcl.loadClass(baseClassName), methodName);
 
-      while (baseClass != null) {
-
-	Vector methods = overrideFactory.getMethods(baseClass, methodName);
-
-	int n = methods.size();
-
-	if (n > 0) {
-	  for (int i = 0; i < n; ++i) {
-	    Method m = (Method) methods.elementAt(i);
-	    Signature s = new Signature(m, overrideFactory);
-	    boolean containsSignature = false;
-	    for (int j = 0; j < overrideFactory.candidates.size(); ++j) 
-	      if (s.equals(overrideFactory.candidates.elementAt(j))) {
-		containsSignature = true;
-		break;
-	      }
-	    if (! containsSignature) 
-	      overrideFactory.candidates.addElement(s);
-	  }
-	}
-	baseClass = baseClass.getSuperclass();
+      for (int i = 0; i < methods.size(); ++i) {
+        Method m = (Method) methods.elementAt(i);
+        Signature s = new Signature(m, overrideFactory);
+        boolean containsSignature = false;
+        for (int j = 0; j < overrideFactory.candidates.size(); ++j) 
+          if (s.equals(overrideFactory.candidates.elementAt(j))) {
+            containsSignature = true;
+            break;
+          }
+        if (! containsSignature) 
+          overrideFactory.candidates.addElement(s);
       }
-      
+
       int n = overrideFactory.candidates.size();
 
       if (n > 0) {
 	String res = "(list ";
 	for (int i = 0; i < n; ++i) {
 	  Signature s = (Signature) overrideFactory.candidates.elementAt(i);
-	  String p = s.getParameters(s.getMethod().getParameterTypes());
+          s.setTruncating(false);
+	  String p = s.getParameters(s.getMethod().getParameterTypes(), false);
+          s.setTruncating(true);
 	  res = res + "\"" + methodName + "(" + p +  ")\" ";
 	}
 	res = res + ")";
@@ -176,9 +201,31 @@ public class MethodOverrideFactory extends MethodFactory
     candidates.removeAllElements();
   }
 
+  /**
+   * Gets the candidate signatures for the class and method
+   * passed as arguments on the command line.
+   *
+   * @param args a <code>String[]</code> value
+   */
   public static void main (String[] args) {
 
-    getCandidateSignatures("jde.ui.design.command.Command","getName");
+    String className = "javax.swing.AbstractAction";
+    String methodName = "actionPerformed";
+    
+    if (args.length == 2) {
+
+      className = args[0];
+      methodName = args[1];
+
+    }
+
+    System.out.println("Class: " + className);
+    System.out.println("Method: " + methodName);
+
+    getCandidateSignatures(className, methodName);
+
+
+    
      
   } // end of main ()
   
@@ -189,8 +236,31 @@ public class MethodOverrideFactory extends MethodFactory
 
 /*
  * $Log: MethodOverrideFactory.java,v $
- * Revision 1.6  2003/01/18 05:48:41  andyp
- * sync to jde 2.3.2
+ * Revision 1.14  2004/03/18 05:28:36  paulk
+ * Now overrides protected as well as public methods and does not override final methods.
+ * Thanks to David Esterkin <desterkin@sciquest.com>.
+ *
+ * Revision 1.13  2004/02/19 06:27:44  paulk
+ * Fixes bug that caused the override wizard to fail to match the method designated by the user to be overriden. Thanks to David Esterkin <desterkin@sciquest.com>.
+ *
+ * Revision 1.12  2003/10/20 05:39:24  paulk
+ * Update the getMethods() method to get all the methods of a class
+ * whether declared by the class or inherited by the class, including
+ * abstract methods.
+ *
+ * Revision 1.11  2003/07/23 04:40:07  paulk
+ * main method now accepts class and method name for testing purpose. Added comment
+ * noting the change in behavior of getDeclaredMethods from JDK 1.3 to JDK 1.4.
+ *
+ * Revision 1.10  2003/05/04 05:50:52  paulk
+ * Document the getCandidateSignatures method.
+ *
+ * Revision 1.9  2003/05/04 05:42:22  paulk
+ * Change getCandidateSignatures to return signatures with parameter
+ * types only (i.e., without parameter nams). This fixes a
+ * signature-matching bug on the Lisp side where the method override
+ * wizard was inadvertently trying to matching a signature without
+ * parameter names to a list of signatures with parameter names.
  *
  * Revision 1.8  2002/06/03 18:12:27  mnl
  * Use DynamicClassLoader instead of simple Class.forName to avoid class
