@@ -14,10 +14,11 @@
 (custom-add-loads 'bbdb-saving '("bbdb"))
 (custom-add-loads 'bbdb-utilities-server '("bbdb-srv"))
 (custom-add-loads 'bbdb-mua-specific-vm '("bbdb-vm"))
+(custom-add-loads 'bbdb-mua-specific-rmail '("bbdb-rmail"))
 (custom-add-loads 'bbdb-utilities-print '("bbdb-print"))
 (custom-add-loads 'bbdb-save '("bbdb"))
 (custom-add-loads 'bbdb-hooks '("bbdb-com" "bbdb"))
-(custom-add-loads 'bbdb-phone-dialing '("bbdb-xemacs" "bbdb-com"))
+(custom-add-loads 'bbdb-phone-dialing '("bbdb-com" "bbdb-xemacs"))
 (custom-add-loads 'bbdb-mua-specific-gnus-splitting '("bbdb-gnus"))
 (custom-add-loads 'bbdb-utilities-finger '("bbdb-com"))
 (custom-add-loads 'bbdb-utilities '("bbdb-whois" "bbdb"))
