@@ -396,9 +396,7 @@ the GDB commands `cd DIR' and `directory'."
 	(output-after-point (< (point) (process-mark proc))))
     (save-excursion
       ;; Insert the text, moving the process-marker.
-      (goto-char (process-mark proc))
-      (insert-before-markers string)
-      (set-marker (process-mark proc) (point))
+      (comint-output-filter proc string)
       (gdb-maybe-delete-prompt)
       ;; Check for a filename-and-line number.
       (gdb-display-frame
