@@ -492,9 +492,27 @@ MODE is the major mode."
 	     (and (null filename)
 		  (memq mode desktop-buffer-modes-to-save))))))
 ;; ----------------------------------------------------------------------------
+(defcustom desktop-relative-file-names nil
+  "*Non-nil means file names in the desktop file are relative to
+`desktop-dirname'."
+  :type 'boolean
+  :group 'desktop)
+
+(defvar desktop-partially-loaded-p nil
+  "For Desktop's internal use.
+
+If non-nil, Desktop was interrupted while loading, and the desktop was
+probably only partially loaded.  Checked by `desktop-save' to avoid
+overwriting a complete desktop file with a partial list.")
+
 (defun desktop-save (dirname)
   "Save the Desktop file.  Parameter DIRNAME specifies where to save desktop."
   (interactive "DDirectory to save desktop file in: ")
+  (when (and desktop-partially-loaded-p
+	     (not (y-or-n-p
+		   "Desktop only partially loaded.  Write file anyway? ")))
+    (error 'invalid-state "partially-loaded desktop -- not saving"
+	   (expand-file-name desktop-basefilename dirname)))
   (run-hooks 'desktop-save-hook)
   (save-excursion
     (let ((filename (expand-file-name desktop-basefilename dirname))
@@ -504,7 +522,11 @@ MODE is the major mode."
 		   (lambda (b)
                      (set-buffer b)
                      (list
-                      (buffer-file-name)
+                      (let ((bn (buffer-file-name)))
+                        (if bn
+                            (if desktop-relative-file-names
+                                (file-relative-name bn dirname)
+                              bn)))
                       (buffer-name)
                       major-mode
                       ;; minor modes
@@ -598,6 +620,7 @@ This is a no-op when Emacs is running in batch mode."
       (setq desktop-dirname (and dirs (expand-file-name (car dirs))))
       (if desktop-dirname
 	  (let ((desktop-last-buffer nil))
+	    (setq desktop-partially-loaded-p t)
 	    ;; `load-with-code-conversion' calls `eval-buffer' which
 	    ;; contains a `save-excursion', so we end up with the same
 	    ;; buffer before and after the load.  This is a problem
@@ -610,6 +633,7 @@ This is a no-op when Emacs is running in batch mode."
 	      (switch-to-buffer desktop-last-buffer))
 	    (run-hooks 'desktop-delay-hook)
 	    (setq desktop-delay-hook nil)
+	    (setq desktop-partially-loaded-p nil)
 	    (message "Desktop loaded."))
 	(desktop-clear)))))
 ;; ----------------------------------------------------------------------------
@@ -704,6 +728,12 @@ to provide correct modes for autoloaded files."
 				  desktop-buffer-major-mode
 				  mim pt mk ro desktop-buffer-misc
 				  &optional locals)
+
+  (if (and desktop-buffer-file-name
+	   (not (file-name-absolute-p desktop-buffer-file-name)))
+      (setq desktop-buffer-file-name
+            (expand-file-name desktop-buffer-file-name desktop-dirname)))
+      
   (let ((hlist desktop-buffer-handlers)
 	(result)
 	(handler))
