@@ -1,9 +1,9 @@
 ;;; eieio-opt.el -- eieio optional functions (debug, printing, speedbar)
 
-;;; Copyright (C) 1996, 1998, 1999, 2000, 2001, 2002 Eric M. Ludlam
+;;; Copyright (C) 1996, 1998, 1999, 2000, 2001, 2002, 2003, 2005 Eric M. Ludlam
 ;;
 ;; Author: <zappo@gnu.org>
-;; RCS: $Id: eieio-opt.el,v 1.3 2002/08/09 21:54:35 youngs Exp $
+;; RCS: $Id: eieio-opt.el,v 1.5 2007-11-26 15:01:05 michaels Exp $
 ;; Keywords: OO, lisp
 ;;                                                                          
 ;; This program is free software; you can redistribute it and/or modify
@@ -17,12 +17,9 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with this program; if not, you can either send email to this
-;; program's author (see below) or write to:
-;;
-;;              The Free Software Foundation, Inc.
-;;              675 Mass Ave.
-;;              Cambridge, MA 02139, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;; Boston, MA 02110-1301, USA.
 ;;
 ;; Please send bug reports, etc. to zappo@gnu.org
 
@@ -35,6 +32,7 @@
 (require 'eieio)
 
 ;;; Code:
+;;;###autoload
 (defun eieio-browse (&optional root-class)
   "Create an object browser window to show all objects.
 If optional ROOT-CLASS, then start with that, otherwise start with
@@ -92,16 +90,16 @@ If CLASS is actually an object, then also display current values of that obect."
 	(princ " Inherits from ")
 	(while pl
 	  (princ "`") (prin1 (car pl)) (princ "'")
-	  (if pl (princ ", "))
-	  (setq pl (cdr pl)))
+	  (setq pl (cdr pl))
+	  (if pl (princ ", ")))
 	(terpri)))
     (let ((ch (class-children class)))
       (when ch
 	(princ " Children ")
 	(while ch
 	  (princ "`") (prin1 (car ch)) (princ "'")
-	  (if ch (princ ", "))
-	  (setq ch (cdr ch)))
+	  (setq ch (cdr ch))
+	  (if ch (princ ", ")))
 	(terpri)))
     (terpri)
     ;; System documentation
@@ -128,6 +126,13 @@ If CLASS is actually an object, then also display current values of that obect."
 	  (princ "'")
 	  (if (not doc)
 	      (princ "  Undocumented")
+	    (if (car doc)
+		(progn
+		  (princ "  :STATIC ")
+		  (prin1 (car (car doc)))
+		  (terpri)
+		  (princ (cdr (car doc)))))
+	    (setq doc (cdr doc))
 	    (if (car doc)
 		(progn
 		  (princ "  :BEFORE ")
@@ -287,7 +292,7 @@ Also extracts information about all methods specific to this generic."
     (terpri)
     (terpri)
     (let ((i 3)
-	  (prefix [ ":BEFORE" ":PRIMARY" ":AFTER" ] ))
+	  (prefix [ ":STATIC" ":BEFORE" ":PRIMARY" ":AFTER" ] ))
       ;; Loop over fanciful generics
       (while (< i 6)
 	(let ((gm (aref (get generic 'eieio-method-tree) i)))
@@ -398,8 +403,9 @@ Optional argument HISTORYVAR is the variable to use as history."
 
 ;;; Help system augmentation
 ;;
-(defun eieio-help-mode-augmentation-maybee ()
-  "For buffers thrown into help mode, augment for eieio."
+(defun eieio-help-mode-augmentation-maybee (&rest unused)
+  "For buffers thrown into help mode, augment for eieio.
+Arguments UNUSED are not used."
   ;; Scan created buttons so far if we are in help mode.
   (when (eq major-mode 'help-mode)
     (save-excursion
@@ -434,7 +440,7 @@ Optional argument HISTORYVAR is the variable to use as history."
 	(while (re-search-forward "^\\(Instance\\|Class\\) Allocated Slots:$" nil t)
 	    (put-text-property (match-beginning 0) (match-end 0) 'face 'bold))
 	(goto-char (point-min))
-	(while (re-search-forward ":\\(BEFORE\\|AFTER\\|PRIMARY\\)" nil t)
+	(while (re-search-forward ":\\(STATIC\\|BEFORE\\|AFTER\\|PRIMARY\\)" nil t)
 	    (put-text-property (match-beginning 0) (match-end 0) 'face 'bold))
 	(goto-char (point-min))
 	(while (re-search-forward "^\\(Private \\)?Slot:" nil t)
@@ -515,7 +521,7 @@ expansion depth."
 				class
 				'speedbar-directory-face
 				depth)
-      (speedbar-make-tag-line 'angle ? nil nil
+      (speedbar-make-tag-line 'angle ?  nil nil
 			      (symbol-name class)
 			      'eieio-describe-class-sb
 			      class
