@@ -1,11 +1,11 @@
 ;;; jde-widgets.el -- Custom-style widgets used by the JDE
-;; $Revision: 1.25 $ $Date: 2003/01/12 23:34:51 $ 
+;; $Revision: 1.26 $ $Date: 2004/06/03 02:21:13 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
 ;; Keywords: java, tools
 
-;; Copyright (C) 1997, 1998, 1999, 2001 Paul Kinnucan.
+;; Copyright (C) 1997-2004 Paul Kinnucan.
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -40,7 +40,8 @@
 (require 'wid-edit)
 (require 'eieio)
 (require 'efc)
-(require 'tree-widget)
+(require 'jde-util)
+(jde-require 'tree-widget)
 
 
 ;; ----------------------------------------------------------------------
@@ -584,6 +585,9 @@ expand button causes the widget to display the values of the array."
 (provide 'jde-widgets)
 
 ;; $Log: jde-widgets.el,v $
+;; Revision 1.26  2004/06/03 02:21:13  paulk
+;; jde-require tree-widget.
+;;
 ;; Revision 1.25  2003/01/12 23:34:51  jslopez
 ;; Small, patch to reduce the amount of cases that cause infinite recursion.
 ;;
