@@ -243,7 +243,8 @@ If optional argument FORCE is non-nil, always update toolbar."
 	     (if (and (featurep 'xpm)
 		      (setq file (liece-locate-icon-file "liece-pointer.xpm")))
 		 (list (vector 'xpm :file file)))
-	     (if (setq file (liece-locate-icon-file "liece-pointer.xbm"))
+	     (if (and (featurep 'xbm)
+		      (setq file (liece-locate-icon-file "liece-pointer.xbm")))
 		 (list (vector 'xbm :file file)))
 	     '([string :data "Liece:"]))))))
     (set-glyph-face glyph 'modeline-buffer-id)
