@@ -6,19 +6,24 @@
 
 (custom-add-loads 'shell '("net-utils"))
 (custom-add-loads 'feedmail-debug '("feedmail"))
-(custom-add-loads 'mail '("feedmail" "metamail" "xemacsbug"))
+(custom-add-loads 'google '("google-query"))
+(custom-add-loads 'mail '("feedmail" "metamail" "mozmail" "xemacsbug"))
 (custom-add-loads 'tools '("rcompile"))
 (custom-add-loads 'feedmail-header '("feedmail"))
-(custom-add-loads 'hypermedia '("metamail"))
+(custom-add-loads 'hypermedia '("google-query" "metamail"))
+(custom-add-loads 'dig '("dig"))
 (custom-add-loads 'feedmail-spray '("feedmail"))
 (custom-add-loads 'feedmail-queue '("feedmail"))
+(custom-add-loads 'comm '("dig" "dns-mode"))
 (custom-add-loads 'remote-compile '("rcompile"))
 (custom-add-loads 'metamail '("metamail"))
 (custom-add-loads 'shadowfile '("shadowfile"))
+(custom-add-loads 'mozmail '("mozmail"))
 (custom-add-loads 'files '("shadowfile"))
 (custom-add-loads 'xemacsbug '("xemacsbug"))
 (custom-add-loads 'feedmail-misc '("feedmail"))
 (custom-add-loads 'maint '("xemacsbug"))
+(custom-add-loads 'dns-mode '("dns-mode"))
 (custom-add-loads 'feedmail '("feedmail"))
 (custom-add-loads 'processes '("metamail" "rcompile"))
 (custom-add-loads 'net-utils '("net-utils"))
