@@ -5,6 +5,6 @@
 (autoload 'custom-add-loads "cus-load")
 
 (custom-add-loads 'files '("tramp"))
-(custom-add-loads 'tramp '("tramp" "tramp-ftp" "tramp-smb"))
+(custom-add-loads 'tramp '("tramp-ftp" "tramp-smb" "tramp"))
 
 ;;; custom-load.el ends here
