@@ -1,22 +1,26 @@
-;;; Sorting and moving messages inside VM
-;;; Copyright (C) 1993, 1994 Kyle E. Jones
-;;;
-;;; This program is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 1, or (at your option)
-;;; any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program; if not, write to the Free Software
-;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+;;; vm-sort.el ---  Sorting and moving messages inside VM
+;; Copyright (C) 1993, 1994 Kyle E. Jones
+;; Copyright (C) 2003-2006 Robert Widhopf-Fenk
+;;
+;; This program is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2 of the License, or
+;; (at your option) any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License along
+;; with this program; if not, write to the Free Software Foundation, Inc.,
+;; 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-(provide 'vm-sort)
 
+;;; Commentary:
+;; 
+
+;;;###autoload
 (defun vm-move-message-forward (count)
   "Move a message forward in a VM folder.
 Prefix arg COUNT causes the current message to be moved COUNT messages forward.
@@ -47,8 +51,8 @@ the folder undisturbed."
     (while (not (zerop count))
       (vm-move-message-pointer direction)
       (vm-decrement count))
-    (if (> (string-to-int (vm-number-of (car vm-message-pointer)))
-	   (string-to-int (vm-number-of (car ovmp))))
+    (if (> (string-to-number (vm-number-of (car vm-message-pointer)))
+	   (string-to-number (vm-number-of (car ovmp))))
 	(setq vm-message-pointer (cdr vm-message-pointer)))
     (if (eq vm-message-pointer ovmp)
 	()
@@ -92,8 +96,8 @@ the folder undisturbed."
 		   vm-numbering-redo-end-point (cdr ovmp-prev)
 		   vm-summary-pointer (car ovmp-prev)))
 	    ((or (not vm-message-pointer)
-		 (< (string-to-int (vm-number-of (car ovmp-prev)))
-		    (string-to-int (vm-number-of (car vm-message-pointer)))))
+		 (< (string-to-number (vm-number-of (car ovmp-prev)))
+		    (string-to-number (vm-number-of (car vm-message-pointer)))))
 	     (setq vm-numbering-redo-start-point (cdr ovmp-prev)
 		   vm-numbering-redo-end-point (cdr ovmp)
 		   vm-summary-pointer (car (cdr ovmp-prev))))
@@ -130,6 +134,7 @@ the folder undisturbed."
       (vm-preview-current-message)
     (vm-update-summary-and-mode-line)))
 
+;;;###autoload
 (defun vm-move-message-backward (count)
   "Move a message backward in a VM folder.
 Prefix arg COUNT causes the current message to be moved COUNT
@@ -144,12 +149,14 @@ the folder undisturbed."
   (interactive "p")
   (vm-move-message-forward (- count)))
 
+;;;###autoload
 (defun vm-move-message-forward-physically (count)
   "Like vm-move-message-forward but always move the message physically."
   (interactive "p")
   (let ((vm-move-messages-physically t))
     (vm-move-message-forward count)))
 
+;;;###autoload
 (defun vm-move-message-backward-physically (count)
   "Like vm-move-message-backward but always move the message physically."
   (interactive "p")
@@ -207,6 +214,7 @@ the folder undisturbed."
        ;; delete the old copy of the message
        (delete-region doomed-start doomed-end)))))
 
+;;;###autoload
 (defun vm-so-sortable-datestring (m)
   (or (vm-sortable-datestring-of m)
       (progn
@@ -217,9 +225,10 @@ the folder undisturbed."
 	      (or (vm-get-header-contents m "Date:")
 		  (vm-grok-From_-date m)
 		  "Thu, 1 Jan 1970 00:00:00 GMT"))
-	   (x-error "1970010100:00:00")))
+	   (error "1970010100:00:00")))
 	(vm-sortable-datestring-of m))))
 
+;;;###autoload
 (defun vm-so-sortable-subject (m)
   (or (vm-sortable-subject-of m)
       (progn
@@ -246,6 +255,12 @@ the folder undisturbed."
 	   subject ))
 	(vm-sortable-subject-of m))))
 
+(defvar vm-sort-compare-header nil
+  "the header to sort on.")
+
+(defvar vm-sort-compare-header-history nil)
+
+;;;###autoload
 (defun vm-sort-messages (keys &optional lets-get-physical)
   "Sort message in a folder by the specified KEYS.
 You may sort by more than one particular message key.  If
@@ -255,11 +270,13 @@ read from the minibuffer.  Valid keys are
 
 \"date\"		\"reversed-date\"
 \"author\"		\"reversed-author\"
+\"full-name\"		\"reversed-full-name\"
 \"subject\"		\"reversed-subject\"
 \"recipients\"		\"reversed-recipients\"
 \"line-count\"		\"reversed-line-count\"
 \"byte-count\"		\"reversed-byte-count\"
 \"physical-order\"	\"reversed-physical-order\"
+\"spam-score\"		\"reversed-spam-score\"
 
 Optional second arg (prefix arg interactively) means the sort
 should change the physical order of the messages in the folder.
@@ -291,7 +308,8 @@ folder in the order in which the messages arrived."
 	doomed-start doomed-end offset
 	(order-did-change nil)
 	virtual
-	physical)
+	physical
+        auto-folder-p)
     (setq key-list (vm-parse keys "[ \t]*\\([^ \t]+\\)")
 	  ml-keys (and key-list (mapconcat (function identity) key-list "/"))
 	  key-funcs nil
@@ -308,10 +326,17 @@ folder in the order in which the messages arrived."
 	     (vm-build-threads-if-unbuilt)
 	     (vm-build-thread-lists)
 	     (setq key-funcs (cons 'vm-sort-compare-thread key-funcs)))
+	    ((equal key "auto-folder")
+             (setq auto-folder-p t)
+             (setq key-funcs (cons 'vm-sort-compare-auto-folder key-funcs)))
 	    ((equal key "author")
 	     (setq key-funcs (cons 'vm-sort-compare-author key-funcs)))
 	    ((equal key "reversed-author")
 	     (setq key-funcs (cons 'vm-sort-compare-author-r key-funcs)))
+	    ((equal key "full-name")
+	     (setq key-funcs (cons 'vm-sort-compare-full-name key-funcs)))
+	    ((equal key "reversed-full-name")
+	     (setq key-funcs (cons 'vm-sort-compare-full-name-r key-funcs)))
 	    ((equal key "date")
 	     (setq key-funcs (cons 'vm-sort-compare-date key-funcs)))
 	    ((equal key "reversed-date")
@@ -332,11 +357,22 @@ folder in the order in which the messages arrived."
 	     (setq key-funcs (cons 'vm-sort-compare-line-count key-funcs)))
 	    ((equal key "reversed-line-count")
 	     (setq key-funcs (cons 'vm-sort-compare-line-count-r key-funcs)))
+	    ((equal key "spam-score")
+	     (setq key-funcs (cons 'vm-sort-compare-spam-score key-funcs)))
+	    ((equal key "reversed-spam-score")
+	     (setq key-funcs (cons 'vm-sort-compare-spam-score-r key-funcs)))
 	    ((equal key "physical-order")
 	     (setq key-funcs (cons 'vm-sort-compare-physical-order key-funcs)))
 	    ((equal key "reversed-physical-order")
 	     (setq key-funcs (cons 'vm-sort-compare-physical-order-r key-funcs)))
-	    (t (error "Unknown key: %s" key)))
+            ((equal key "header")
+             (setq vm-sort-compare-header nil)
+             (setq key-funcs (cons 'vm-sort-compare-header key-funcs)))
+	    (t
+             (let ((compare (intern (format "vm-sort-compare-%s" key))))
+               (if (functionp compare)
+                   (setq key-funcs (cons compare key-funcs))
+                 (error "Unknown key: %s" key)))))
       (setq key-list (cdr key-list)))
     ;; if this is not a thread sort and threading is enabled,
     ;; then disable threading and make sure the whole summary is
@@ -386,7 +422,7 @@ folder in the order in which the messages arrived."
 		(vm-set-su-start-of (car mp-new)
 				    (vm-su-start-of (car mp-old)))))
 	  (setq mp-new nil)))
-      (if (and order-did-change physical)
+      (if (and physical (vm-has-message-order))
 	  (let ((buffer-read-only nil))
 	    ;; the folder is being physically ordered so we don't
 	    ;; need a message order header to be stuffed, nor do
@@ -446,8 +482,12 @@ folder in the order in which the messages arrived."
 	     (or lets-get-physical vm-move-messages-physically))
 	;; clip region is most likely messed up
 	(vm-preview-current-message)
-      (vm-update-summary-and-mode-line))))
+      (vm-update-summary-and-mode-line))
 
+    (if auto-folder-p
+        (vm-sort-insert-auto-folder-names))))
+
+;;;###autoload
 (defun vm-sort-compare-xxxxxx (m1 m2)
   (let ((key-funcs vm-key-functions) result)
     (while (and key-funcs
@@ -500,6 +540,20 @@ folder in the order in which the messages arrived."
 	  ((string-equal s1 s2) '=)
 	  (t t))))
 
+(defun vm-sort-compare-full-name (m1 m2)
+  (let ((s1 (vm-su-full-name m1))
+	(s2 (vm-su-full-name m2)))
+    (cond ((string-lessp s1 s2) t)
+	  ((string-equal s1 s2) '=)
+	  (t nil))))
+
+(defun vm-sort-compare-full-name-r (m1 m2)
+  (let ((s1 (vm-su-full-name m1))
+	(s2 (vm-su-full-name m2)))
+    (cond ((string-lessp s1 s2) nil)
+	  ((string-equal s1 s2) '=)
+	  (t t))))
+
 (defun vm-sort-compare-date (m1 m2)
   (let ((s1 (vm-so-sortable-datestring m1))
 	(s2 (vm-so-sortable-datestring m2)))
@@ -543,33 +597,48 @@ folder in the order in which the messages arrived."
 	  (t t))))
 
 (defun vm-sort-compare-line-count (m1 m2)
-  (let ((n1 (string-to-int (vm-su-line-count m1)))
-	(n2 (string-to-int (vm-su-line-count m2))))
+  (let ((n1 (string-to-number (vm-su-line-count m1)))
+	(n2 (string-to-number (vm-su-line-count m2))))
     (cond ((< n1 n2) t)
 	  ((= n1 n2) '=)
 	  (t nil))))
 
 (defun vm-sort-compare-line-count-r (m1 m2)
-  (let ((n1 (string-to-int (vm-su-line-count m1)))
-	(n2 (string-to-int (vm-su-line-count m2))))
+  (let ((n1 (string-to-number (vm-su-line-count m1)))
+	(n2 (string-to-number (vm-su-line-count m2))))
     (cond ((> n1 n2) t)
 	  ((= n1 n2) '=)
 	  (t nil))))
 
 (defun vm-sort-compare-byte-count (m1 m2)
-  (let ((n1 (string-to-int (vm-su-byte-count m1)))
-	(n2 (string-to-int (vm-su-byte-count m2))))
+  (let ((n1 (string-to-number (vm-su-byte-count m1)))
+	(n2 (string-to-number (vm-su-byte-count m2))))
     (cond ((< n1 n2) t)
 	  ((= n1 n2) '=)
 	  (t nil))))
 
 (defun vm-sort-compare-byte-count-r (m1 m2)
-  (let ((n1 (string-to-int (vm-su-byte-count m1)))
-	(n2 (string-to-int (vm-su-byte-count m2))))
+  (let ((n1 (string-to-number (vm-su-byte-count m1)))
+	(n2 (string-to-number (vm-su-byte-count m2))))
     (cond ((> n1 n2) t)
 	  ((= n1 n2) '=)
 	  (t nil))))
 
+(defun vm-sort-compare-spam-score (m1 m2)
+  (let ((s1 (vm-su-spam-score m1))
+	(s2 (vm-su-spam-score m2)))
+    (cond ((< s1 s2) t)
+	  ((= s1 s2) '=)
+	  (t nil))))
+
+(defun vm-sort-compare-spam-score-r (m1 m2)
+  (let ((s1 (vm-su-spam-score m1))
+	(s2 (vm-su-spam-score m2)))
+    (cond ((< s1 s2) nil)
+	  ((= s1 s2) '=)
+	  (t t))))
+
+;;;###autoload
 (defun vm-sort-compare-physical-order (m1 m2)
   (let ((n1 (vm-start-of m1))
 	(n2 (vm-start-of m2)))
@@ -577,9 +646,43 @@ folder in the order in which the messages arrived."
 	  ((= n1 n2) '=)
 	  (t nil))))
 
+;;;###autoload
 (defun vm-sort-compare-physical-order-r (m1 m2)
   (let ((n1 (vm-start-of m1))
 	(n2 (vm-start-of m2)))
     (cond ((> n1 n2) t)
 	  ((= n1 n2) '=)
 	  (t nil))))
+
+(add-to-list 'vm-supported-sort-keys "header")
+
+(defun vm-get-headers-of (m &optional headers)
+  (save-excursion
+    (save-restriction
+      (widen)
+     (let ((end (vm-text-of m)))
+       (set-buffer (vm-buffer-of m))
+       (goto-char (vm-start-of m))
+       (while (re-search-forward "^[^: \n\t]+:" end t)
+         (add-to-list 'headers (match-string 0)))
+       headers))))
+
+(defun vm-sort-compare-header (m1 m2)
+  (if (null vm-sort-compare-header)
+      (setq vm-sort-compare-header
+            (completing-read
+             (if (car vm-sort-compare-header-history)
+                 (format "Sort hy header (%s): "
+                         (car vm-sort-compare-header-history))
+               "Sort hy header: ")
+             (mapcar (lambda (h) (list h))
+                     (vm-get-headers-of m2 (vm-get-headers-of m1)))
+             nil nil nil
+             'vm-sort-compare-header-history
+             (car vm-sort-compare-header-history)))
+    (string< (vm-get-header-contents m1 vm-sort-compare-header)
+             (vm-get-header-contents m2 vm-sort-compare-header))))
+
+(provide 'vm-sort)
+
+;;; vm-sort.el ends here
