@@ -133,6 +133,18 @@
 
   )
 
+(defsubst set-font-style-by-keywords (fontobj styles)
+  (make-local-variable 'font-func)
+  (declare (special font-func))
+  (if (listp styles)
+      (while styles
+	(setq font-func (car-safe (cdr-safe (assq (car styles) font-style-keywords)))
+	      styles (cdr styles))
+	(and (fboundp font-func) (funcall font-func fontobj t)))
+    (setq font-func (car-safe (cdr-safe (assq styles font-style-keywords))))
+    (and (fboundp font-func) (funcall font-func fontobj t))))
+
+
 (defvar w3-display-same-buffer nil)
 (defvar w3-face-cache nil  "Cache for w3-face-for-element")
 (defvar w3-face-index 0)
@@ -174,7 +186,7 @@
   (save-excursion
     (goto-char (or cur-viewing-pos (point-min)))
     (cond
-     (w3-running-xemacs
+     ((featurep 'xemacs)
       (if (and (not (sit-for 0)) (input-pending-p))
 	  (condition-case ()
 	      (let ((buffer-read-only t))
@@ -273,7 +285,7 @@
   (equal foreground background))
 
 (defun w3-get-default-color (backgroundp)
-  (if w3-running-xemacs
+  (if (featurep 'xemacs)
       (or (if backgroundp
 	      (face-background-name 'default)
 	    (face-foreground-name 'default))
@@ -339,7 +351,7 @@ If the face already exists, it is unmodified."
 (cond
  ((not (fboundp 'make-face))
   (fset 'w3-make-face 'ignore))
- (w3-running-xemacs
+ ((featurep 'xemacs)
   (fset 'w3-make-face 'make-face))
  (t
   (fset 'w3-make-face 'w3-make-face-emacs19)))
@@ -916,7 +928,7 @@ If the face already exists, it is unmodified."
 	  (not (buffer-name buffer)))
       nil)
      ((and (eq widget 'background)
-	   w3-running-xemacs)
+	   (featurep 'xemacs))
       (set-face-background-pixmap face
 				  (glyph-image-instance glyph)
 				  buffer))
@@ -1987,7 +1999,7 @@ Format: (((image-alt row column) . offset) ...)")
 (defvar w3-display-hackmap nil "Keymap used for hyperlink widgets")
   
 (defun w3-resurrect-hyperlinks ()
-  (if (and (not w3-display-hackmap) w3-running-xemacs)
+  (if (and (not w3-display-hackmap) (featurep 'xemacs))
       (progn
 	(setq w3-display-hackmap (make-sparse-keymap))
 	(set-keymap-parent w3-display-hackmap widget-button-keymap)
@@ -2437,7 +2449,7 @@ Format: (((image-alt row column) . offset) ...)")
 		 (setf (car w3-face-color) fore)
 		 (setf (car w3-face-background-color) back)
 
-		 (if (not font-running-xemacs)
+		 (if (not (featurep 'xemacs))
 		     (setq w3-display-background-properties (cons fore back))
 		   (if pixm
 		       (w3-maybe-start-background-image-download pixm 'default))
@@ -2714,7 +2726,7 @@ Format: (((image-alt row column) . offset) ...)")
 	(push-mark (point) t)
 	(w3-find-specific-link (url-target url-current-object)))
     (goto-char (point-min)))
-  (and (not w3-running-xemacs)
+  (and (not (featurep 'xemacs))
        (not (eq (device-type) 'tty))
        (w3-fixup-eol-faces))
   (message "Drawing... done"))
